//
// ListViewEntry.cpp
// Implementation for TListViewEntry class
//

#include <stdio.h>
#include <string>
#include "ListViewEntry.h"

enum {
	PMI_ITEMS = 'itmz'
};

//
// Property Editors for ListView properties
//
class TListViewTypeProperty: public TPickListProperty {
public:
	TListViewTypeProperty(TComponent *Component, TProperty *prop)
	: TPickListProperty(Component, prop)
	{}
	
	virtual void GetValuePickList(TStringList *list)
	{
		list->MakeEmpty();
		list->AddItem("B_SINGLE_SELECTION_LIST");
		list->AddItem("B_MULTIPLE_SELECTION_LIST");
	}
};

class TListViewItemsWin: public BWindow {
public:
	enum {
		EFVALUE_CHANGED = 0x1
	, BTNADD_INVOKED
	, BTNEDIT_INVOKED
	, BTNDEL_INVOKED
	, BTNOK_INVOKED
	, BTNCANCEL_INVOKED
	, LVITEMS_SELECTED
	, LVITEMS_INVOKED
	};
	
	TDesignListView *m_ListView;
	
	TListViewItemsWin(TDesignListView *lv)
	: BWindow(BRect(100,100,350,370), "ListView Items", B_TITLED_WINDOW, B_NOT_ZOOMABLE | B_NOT_RESIZABLE)
	, m_ListView(lv)
	{
		char buf[80] = "";
		sprintf(buf, "%s Items", lv->ComponentName());
		SetTitle(buf);
		SetFeel(B_MODAL_APP_WINDOW_FEEL);
		
		boxPanel = new BBox(Bounds());
		BRect r = Bounds();
		r.InsetBy(8,8);
		r.bottom -= 30;
		boxItems = new BBox(r, "boxItems");
		boxItems->SetLabel("Items:");
		
		r = boxItems->Bounds();
		r.InsetBy(10,10);
		r.bottom -= 60;
		r.top += 5;
		r.right -= 15;
		lvItems = new BListView(r,"lvItems",B_SINGLE_SELECTION_LIST);
		lvItems->SetSelectionMessage(new BMessage(LVITEMS_SELECTED));
		lvItems->SetInvocationMessage(new BMessage(LVITEMS_INVOKED));
		
		AddChild(boxPanel);
		boxPanel->AddChild(boxItems);
		boxItems->AddChild(
			new BScrollView(
				"svItems", lvItems, B_FOLLOW_ALL, 0, false, true
			)
		);
		
		r.top = lvItems->Frame().bottom + 25;
		r.bottom = r.top + 30;
		r.left = 10;
		r.right = boxItems->Bounds().Width() - 20;
		efValue = new BTextControl(r,"efValue","Item value:","",new BMessage(EFVALUE_CHANGED));
		boxItems->AddChild(efValue);
		
		r.top += 25; r.bottom = r.top + 25;
		r.left = 10; r.right = r.left + 45;
		btnAdd = new BButton(r, "btnAdd", "Add", new BMessage(BTNADD_INVOKED));
		boxItems->AddChild(btnAdd);

		r.left += 50; r.right += 50;
		btnEdit = new BButton(r, "btnEdit", "Edit", new BMessage(BTNEDIT_INVOKED));
		boxItems->AddChild(btnEdit);

		r.left += 50; r.right += 50;
		btnDelete = new BButton(r, "btnDelete", "Delete", new BMessage(BTNDEL_INVOKED));
		boxItems->AddChild(btnDelete);
		
		btnAdd->SetEnabled(true);
		btnEdit->SetEnabled(false);
		btnDelete->SetEnabled(false);
		
		r.top = boxItems->Frame().bottom + 6;
		r.bottom = r.top + 25;
		r.left = (Bounds().Width()-140);
		r.right = Bounds().Width()-80;
		btnOK = new BButton(r, "btnOK", "Save", new BMessage(BTNOK_INVOKED)); 
		
		r.left += 70;
		r.right += 70;
		btnCancel = new BButton(r, "btnCancel", "Close", new BMessage(BTNCANCEL_INVOKED));

		boxPanel->AddChild(btnOK);
		boxPanel->AddChild(btnCancel);
		
		lvItems->SetTarget(this);
		btnAdd->SetTarget(this);
		btnEdit->SetTarget(this);
		btnDelete->SetTarget(this);
		btnOK->SetTarget(this);
		btnCancel->SetTarget(this);
		efValue->SetTarget(this);
		
		// Insert the currently-stored items in the listview.
		lvItems->MakeEmpty();
		for(int32 i = 0; i < m_ListView->CountItems(); i++)
		{
			BStringItem *item = dynamic_cast<BStringItem *>(m_ListView->ItemAt(i));
			lvItems->AddItem(new BStringItem(item->Text()));
		}
		
		Show();
	}
	
	virtual void MessageReceived(BMessage *message)
	{
		switch(message->what)
		{
			case EFVALUE_CHANGED:
			{
				if(0 < strlen(efValue->Text()))
				{
					BStringItem *item = dynamic_cast<BStringItem *>(lvItems->ItemAt(lvItems->CurrentSelection()));
					
					if(item)
					{
						item->SetText(efValue->Text());
						lvItems->Invalidate();
					}
					else
					{
						lvItems->AddItem(new BStringItem(efValue->Text()));
					}
				}
				break;
			}
			
		 	case BTNADD_INVOKED:
			{
				if(0 < strlen(efValue->Text()))
				{
					lvItems->AddItem(new BStringItem(efValue->Text()));
				}
				break;
			}
			
		 	case BTNEDIT_INVOKED:
			{
				if(0 < strlen(efValue->Text()))
				{
					BStringItem *item = dynamic_cast<BStringItem *>(lvItems->ItemAt(lvItems->CurrentSelection()));
					
					if(item)
					{
						item->SetText(efValue->Text());
						lvItems->Invalidate();
					}
				}
				break;
			}
			
		 	case BTNDEL_INVOKED:
			{
				BStringItem *item = dynamic_cast<BStringItem *>(lvItems->ItemAt(lvItems->CurrentSelection()));
				
				if(item)
				{
					lvItems->RemoveItem(item);
				}
				break;
			}
			
		 	case BTNOK_INVOKED:
			{
				if(m_ListView->DTView()->LockLooper())
				{
					m_ListView->MakeEmpty();
					
					for(int32 i = 0; i < lvItems->CountItems(); i++)
					{
						BStringItem *item = dynamic_cast<BStringItem *>(lvItems->ItemAt(i));
						m_ListView->AddItem(new BStringItem(item->Text()));
					}
					m_ListView->DTView()->UnlockLooper();
				}
				break;
			}
			
		 	case BTNCANCEL_INVOKED:
			{
				Quit();
				break;
			}
			
			case LVITEMS_SELECTED:
			{
				BStringItem *item = dynamic_cast<BStringItem *>(lvItems->ItemAt(lvItems->CurrentSelection()));
				if(item)
				{
					efValue->SetText(item->Text());
				}
				
				btnEdit->SetEnabled(item != 0);
				btnDelete->SetEnabled(item != 0);
				break;
			}
			
		 	case LVITEMS_INVOKED:
			{
				efValue->MakeFocus();
				break;
			}
		
			default:
			{
				BWindow::MessageReceived(message);
				break;
			}
		}
	}
	
	BBox *boxPanel, *boxItems;
	BListView *lvItems;
	BButton 
		*btnAdd
	, *btnEdit
	, *btnDelete
	,	*btnOK
	, *btnCancel
	;
	BTextControl *efValue;
};

//
// TListViewEntry implementation
//
TComponent *TListViewEntry::CreateComponent(BView *view, TFormDesigner *designer, BRect rect)
{
	TDesignListView *btn = new TDesignListView(rect, "ListView", "ListView", designer);
	btn->SetClassName(ClassName());
	
//	BScrollView *scr = new BScrollView(
//		"svListView", btn, B_FOLLOW_LEFT|B_FOLLOW_TOP, 0, false, true
//	);
	
	// default view properties
	view->LockLooper();
//	btn->SetDTView(scr);
//	view->AddChild(scr);
	view->AddChild(btn);
	view->UnlockLooper();
	return btn;
}

//
// TDesignListView implementation
//
TDesignListView::TDesignListView(BRect frame, const char *name, const char *caption, TFormDesigner *designer)
:	BListView(frame, name, B_SINGLE_SELECTION_LIST)
, TControlComponent(0, designer, ctVisual)
{
	SetComponentName(name);
	SetDTView(this); // ALWAYS do this
	InitPropertyList(); // and this
	InitPropertyValues();
	InitEventList();
	InitEventValues();
	
	AddItem(new BStringItem("Item 1"));
	AddItem(new BStringItem("Item 2"));
}

void TDesignListView::MouseDown(BPoint pt)
{
	DesignerMouseDown(pt, Bounds(), this);
	Invalidate();
}

void TDesignListView::Draw(BRect updateRect)
{
	BListView::Draw(updateRect);
	DesignerDrawSelection();
}

bool TDesignListView::AddComponentContextMenuItems(BPopUpMenu *menu)
{
	menu->AddSeparatorItem();
	menu->AddItem(new BMenuItem("Items...", new BMessage(PMI_ITEMS)));
	return true;
}

bool TDesignListView::HandleComponentContextMessage(BMessage *message)
{
	switch(message->what)
	{
		case PMI_ITEMS:
		{
			new TListViewItemsWin(this);
			return true;
		}
	}
	return false;
}

bool TDesignListView::DefaultEditAction(BPoint /*pt*/)
{
	HandleComponentContextMessage(new BMessage(PMI_ITEMS));
	return true;
}

const char *TDesignListView::GetPropertyValue(const char *propName) const
{
	string s = propName;
	
	return TControlComponent::GetPropertyValue(propName);
}

void TDesignListView::SetPropertyValue(const char *propName, const char *value)
{
	printf("TDesignListView::SetPropertyValue('%s', '%s')\n", propName, value);
	if((!propName) || (!value)) return;
	
	string s = propName;
	string v = value;
	
	if(s == "ListType")
	{
		if(DTView()->LockLooper())
		{
			list_view_type type = (v == "B_MULTIPLE_SELECTION_LIST")? B_MULTIPLE_SELECTION_LIST: B_SINGLE_SELECTION_LIST;
			SetListType(type);
			DTView()->UnlockLooper();
		}
		StoreProperty(propName, value);
		return;
	}
	
	TControlComponent::SetPropertyValue(propName, value);
}

void TDesignListView::InitPropertyList(void)
{
	TControlComponent::InitPropertyList();
	
	RemoveProperty(FindProperty("Enabled"));
	
	AddPropAndEditor(
		new TProperty(this, "ListType", "B_SINGLE_SELECTION_LIST")
	, new TListViewTypeProperty(this, 0)
	);
	
	RemoveProperty(FindProperty("Label"));
}

void TDesignListView::InitEventList(void)
{
	TControlComponent::InitEventList();
	
	AddEvent(new TEvent("OnItemSelected", "int32 index"));
	AddEvent(new TEvent("OnItemInvoked", "int32 index"));
}

void TDesignListView::InitPropertyValues(void)
{
	TControlComponent::InitPropertyValues();
	
	SetPropertyValue("ViewColor", "{255,255,255,0}");
}

void TDesignListView::InitEventValues(void)
{
	TControlComponent::InitEventValues();
}

void TDesignListView::OnSetConstructorParams(BString &s)
{
	TControlComponent::OnSetConstructorParams(s);	
	
	char buf[256] = "";
	sprintf(
		buf
	, "%s"
	, FindProperty("ListType")->Value()
	);
	s += buf;
}

void TDesignListView::WriteExtendedComponentDataTo(BMessage *compInfo)
{
	BMessage msg('itmz');
	
	for(int32 i = 0; i < CountItems(); i++)
	{
		BStringItem *item = dynamic_cast<BStringItem *>(ItemAt(i));
		msg.AddString(item->Text(), item->Text());
	}
	
	compInfo->AddMessage("_Items", &msg);
}

void TDesignListView::ReadExtendedComponentDataFrom(BMessage *compInfo)
{
	BMessage msg('itmz');
	if(B_OK == compInfo->FindMessage("_Items", &msg))
	{
		if(DTView()->LockLooper())
		{
			MakeEmpty();
			DTView()->UnlockLooper();
		}
		
		char *name;
		uint32 type;
		int32 count;

		for(int32 i = 0; 
			(B_OK == msg.GetInfo(B_STRING_TYPE, i, &name, &type, &count)); 
			i++)
		{
			const char *value;
			if(B_OK == msg.FindString(name, &value))
			{
				if(DTView()->LockLooper())
				{
					AddItem(new BStringItem(value));
					DTView()->UnlockLooper();
				}
			}
		}
	}
}

bool TDesignListView::OnGenInitForm(BString &s)
{
	for(int32 i = 0; i < CountItems(); i++) 
	{
		BStringItem *item = dynamic_cast<BStringItem *>(ItemAt(i));
		if(item)
		{
			s
			<< "$COMPONENT->AddItem(new BStringItem(\"" << item->Text() << "\"));\n"
			;
		}
	}
	return true;
}

// eof
