//
// OutlineListViewEntry.h
//
#ifndef _OutlineListViewEntry_H
#define _OutlineListViewEntry_H

#include "ComponentDesign.h"
#include "PaletteEntry.h"
#include <List.h>
#include <OutlineListView.h>

//
// Design-mode version of the TButton class: inherits from the
// actual control class to get the control, and from TVisualComponent
// to get the design-time behaviour.
//
class _EXPORT TDesignOutlineListView: public BOutlineListView, public TControlComponent {
public:
	TDesignOutlineListView(BRect, const char *, const char *, TFormDesigner *);
	
	// Be events
	virtual void MouseDown(BPoint);
	virtual void Draw(BRect);
	virtual bool AddComponentContextMenuItems(BPopUpMenu *);
	virtual bool HandleComponentContextMessage(BMessage *);
	virtual bool DefaultEditAction(BPoint pt);
	
	virtual const char *GetPropertyValue(const char *propName) const;
	// Set the property value (converting from text if nec.) for the property
	// referenced by "propName"
	virtual void SetPropertyValue(const char *propName, const char *value);
		
	//
	// These functions should be called from derived classes' constructors.
	//
	// Load the list of properties for this component
	virtual void InitPropertyList(void);
	// Load the list of events for this component
	virtual void InitEventList(void);
	// Initialize design-time property values
	virtual void InitPropertyValues(void);
	// Initialize design-time event values
	virtual void InitEventValues(void);	

	// Code generation
	virtual void OnSetConstructorParams(BString &s);
	virtual const char *HeaderFile(void) const { return "BaseControls.h"; }
	virtual void WriteExtendedComponentDataTo(BMessage *);
	virtual void ReadExtendedComponentDataFrom(BMessage *);
	virtual bool OnGenInitForm(BString &);
};

class _EXPORT TOutlineListViewEntry: public TPaletteEntry {
public:
	TOutlineListViewEntry(image_id imgID)
	: TPaletteEntry(imgID, "OutlineListView", "TOutlineListView")
	{}
	
  virtual TComponent *CreateComponent(BView *, TFormDesigner *, BRect);
  
  virtual const char *Copyright(void) const {return "(C) 1999, HamPilotSoft"; }
};


#endif // _OutlineListViewEntry_H
