//
// StringViewEntry.cpp
// Implementation for TStringViewEntry class
//

#include <stdio.h>
#include <string>
#include "StringViewEntry.h"

enum {
	PMI_ABOUT = 'ham!'
};

//
// property editors for StringView controls
//
class TSVAlignmentProperty: public TPickListProperty {
public:
	TSVAlignmentProperty(TComponent *Component, TProperty *prop)
	: TPickListProperty(Component, prop)
	{}
	
	virtual void GetValuePickList(TStringList *list)
	{
		list->MakeEmpty();
		list->AddItem("B_ALIGN_LEFT");
		list->AddItem("B_ALIGN_RIGHT");
		list->AddItem("B_ALIGN_CENTER");
	}
};


//
// TStringViewEntry implementation
//
TComponent *TStringViewEntry::CreateComponent(BView *view, TFormDesigner *designer, BRect rect)
{
	TDesignStringView *StringView = new TDesignStringView(rect, "StringView", "StringView", designer);
	StringView->SetClassName(ClassName());
	
	view->LockLooper();
	view->AddChild(StringView);
	view->UnlockLooper();
	return StringView;
}

//
// TDesignStringView implementation
//
TDesignStringView::TDesignStringView(BRect frame, const char *name, const char *caption, TFormDesigner *designer)
:	BStringView(frame, name, "StringView")
, TControlComponent(0, designer, ctVisual)
{
	SetComponentName(name);
	SetDTView(this); // ALWAYS do this
	InitPropertyList(); // and this
	InitPropertyValues();
	InitEventList();
	InitEventValues();
}

void TDesignStringView::MouseDown(BPoint pt)
{
	BStringView::MouseDown(pt);
	DesignerMouseDown(pt, Bounds(), this);
	Invalidate();
}

void TDesignStringView::Draw(BRect updateRect)
{
	BStringView::Draw(updateRect);
	DesignerDrawSelection();
}

bool TDesignStringView::AddComponentContextMenuItems(BPopUpMenu *menu)
{
	return true;
}

bool TDesignStringView::HandleComponentContextMessage(BMessage *message)
{
	switch(message->what)
	{
		default: 
			return false;
	}
}

bool TDesignStringView::DefaultEditAction(BPoint pt)
{
	return false;
}

const char *TDesignStringView::GetPropertyValue(const char *propName) const
{
	string s = propName;
	
	if(s == "Text")
	{
		return Text();
	}
	
	return TControlComponent::GetPropertyValue(propName);
}

void TDesignStringView::SetPropertyValue(const char *propName, const char *value)
{
	printf("TDesignStringView::SetPropertyValue('%s', '%s')\n", propName, value);
	if((!propName) || (!value)) return;
	
	string s = propName;
	string v = value;
	
	if(s == "Text")
	{
		if(DTView()->LockLooper())
		{
			SetText(value);
			DTView()->UnlockLooper();
		}
		StoreProperty(propName, value);
		return;
	}
	
	if(s == "Alignment")
	{
		if(DTView()->LockLooper())
		{
			if(v == "B_ALIGN_RIGHT")
			{
				SetAlignment(B_ALIGN_RIGHT);
			}
			else if(v == "B_ALIGN_CENTER")
			{
				SetAlignment(B_ALIGN_CENTER);
			}
			else
			{
				SetAlignment(B_ALIGN_LEFT);
			}
			DTView()->UnlockLooper();
		}
		StoreProperty(propName, value);
		return;
	}
	
	TControlComponent::SetPropertyValue(propName, value);
}

void TDesignStringView::InitPropertyList(void)
{
	TControlComponent::InitPropertyList();
	
	AddPropAndEditor(
		new TProperty(this, "Text", "StringView")
	, new TStringProperty(this, 0)
	);
	
	AddPropAndEditor(	
		new TProperty(this, "Alignment", "B_ALIGN_LEFT")
	, new TSVAlignmentProperty(this, 0)	
	);
	
	RemoveProperty(FindProperty("Enabled"));
	RemoveProperty(FindProperty("Label"));
	
	FindProperty("Text")->SetQuoted(true);
}

void TDesignStringView::InitEventList(void)
{
	TControlComponent::InitEventList();
}

void TDesignStringView::InitPropertyValues(void)
{
	TControlComponent::InitPropertyValues();
}

void TDesignStringView::InitEventValues(void)
{
	TControlComponent::InitEventValues();
}

void TDesignStringView::MessageReceived(BMessage *message)
{
	if(!HandleComponentDesignMessage(message))
	{
		BStringView::MessageReceived(message);
	}
}

void TDesignStringView::OnSetConstructorParams(BString &s)
{
	TControlComponent::OnSetConstructorParams(s);	
	
	char buf[256] = "";
	sprintf(buf, "\"%s\"", FindProperty("Text")->Value());
	s += buf;
}


// eof
